@@ -39,6 +39,24 @@ class DataService:
         pass
 '''
 
+PropertyExtension = '''
+class PropertyExtension: 
+    # 属性名称
+    def name(self):
+        pass
+    # 属性类型
+    def type(self):
+        pass
+    # 属性编辑
+    # 返回值为编辑后的属性值
+    def edit(self, value):
+        pass
+    # 获取属性值的展示文本
+    # 某些场景下，属性值与属性值的显示不完全一样
+    def display_value(self, value):
+        pass
+'''
+
 
 
 class DataServiceManage:
@@ -184,6 +202,10 @@ config = {
         {
             'name': "Core::DataNode::Reg",
             'define': DSREG.DataNodeExtension
+        },
+        {
+            'name': "Core::Data::Property",
+            'define': PropertyExtension
         }
     ],
     "extensions" : [
@@ -260,5 +282,9 @@ config = {
             'name': 'event::DataNode::Select',
             'value1': 'DataNode'
         },
+        {
+            'name': 'event::DataNode::Select::items',
+            'value1': 'DataNodeList'
+        }
     ]
 }
\ No newline at end of file
