@@ -19,90 +19,7 @@ def init(context_):
     context = context_
 
 
-class DataExploreView(QDockWidget):
-    def __init__(self):
-        super().__init__()
-
-        self._open = None
-        self._menu = None
-
-        self.treeWidget = QTreeWidget()
-        self.treeWidget.setContextMenuPolicy(Qt.CustomContextMenu)
-        self.treeWidget.setColumnCount(2)
-        self.treeWidget.setHeaderLabels(['Name', 'Kind'])
-        self.treeWidget.setColumnWidth(0, 160)
-
-        # 右键菜单
-        self.treeWidget.customContextMenuRequested.connect(self.openMenu)
-        # 双击编辑名称
-        self.treeWidget.itemDoubleClicked.connect(self.double_click)
-        # 选择改变
-        self.treeWidget.itemPressed.connect(self.itemPressed)
-
-        self.setWidget(self.treeWidget)
-    def openMenu(self, position):
-        item = self.treeWidget.currentItem()
-
-        # 弹出菜单
-        menu = QMenu()
-        self._menu = menu
-        
-        # 获取类型节点注册的菜单
-        node = None
-        if item:
-            node = item.data(0, Qt.UserRole)
-        exts = context.find_extension("PL::DataExplore::Menu")
-        sort_exts = sorted(exts, key=lambda ext: ext.priorty())
-        for ext in sort_exts:
-            ext.add_menu(menu, node)
-
-        menu.exec_(self.treeWidget.viewport().mapToGlobal(position))
-
-    def double_click(self, item, col):
-        node = item.data(0, Qt.UserRole)
-        
-        exts = context.find_extension("PL::Basic::Editor")
-        for ext in exts:
-            if (ext.is_match_data(node)):
-                global _frame
-                tabWidget = _frame.centralWidget ()
-                widget = ext.create_editor(tabWidget)
-                widget.on_init(node)
-                tabWidget.addTab(widget, node.get_name())
-                break
-
-    def itemPressed(self, item, col):
-        if self._open:
-            item = self._open
-            self.treeWidget.closePersistentEditor(self._open,0)
-            self._open = None
-
-            node = item.data(0, Qt.UserRole)
-            print("name.....:", item.text(0))
-            node.set_name(item.text(0))
-        
-        global context
-        context.fire('event::DataNode::Select', item.data(0, Qt.UserRole))
-
-    def close_project(self):
-        global context
-        print("Close project......................")
-        if self.treeWidget.topLevelItemCount() > 0:
-            self.treeWidget.takeTopLevelItem(0)
-
-    def _new_node(self, node, parent = None):
-        global context
-
-        new_item = None
-        if not parent:
-            new_item = QTreeWidgetItem(self.treeWidget)
-        else:
-            new_item = QTreeWidgetItem(parent)
-            print("parent:",new_item.parent())
-        
-        
-
-        class RootNodeCallback:
+class RootNodeCallback:
             def __init__(self, item, node):
                 self._item = item
                 self._node = node
@@ -144,8 +61,8 @@ class DataExploreView(QDockWidget):
 
                 item_.setExpanded(True)
 
-                print("........................................")
-                print(child.get_children())
+                #print("........................................")
+                #print(child.get_children())
                 for child_node in child.get_children():
                     self.add_exist_child(item, child, child_node)
 
@@ -153,12 +70,22 @@ class DataExploreView(QDockWidget):
                 if node != self._node:
                     return
                 
-                item = QTreeWidgetItem(self._item)
+                parent = child.parent()
+                children = parent.get_children()
+                index = 0
+                for child_node in children:
+                    if child == child_node:
+                        break
+                    index += 1
+
+                item = QTreeWidgetItem()
                 item.setText(0, child.get_name())
                 item.setText(1, child.get_kind())
                 item.setData(0, Qt.UserRole, child)
                 #print("current:", item)
                 #print("parent:", item.parent())
+                #print("insert item: ", index, item)
+                self._item.insertChild(index, item)
 
                 global context
                 obj = RootNodeCallback(item , child)
@@ -172,8 +99,8 @@ class DataExploreView(QDockWidget):
 
                 self._item.setExpanded(True)
 
-                print("........................................")
-                print(child.get_children())
+                #print("........................................")
+                #print(child.get_children())
                 for child_node in child.get_children():
                     self.add_exist_child(item, child, child_node)
             
@@ -188,6 +115,289 @@ class DataExploreView(QDockWidget):
                         self._item.removeChild(item)
                         break
 
+class DragTreeWidget(QTreeWidget):
+    def __init__(self):
+        super().__init__()
+
+    def selectionChanged(self, select, deselect):
+        global context
+
+        items = self.selectedItems()
+        if len(items) == 1:
+            context.fire('event::DataNode::Select', items[0].data(0, Qt.UserRole))
+        elif len(items) > 1:
+            items_nodes = [item.data(0, Qt.UserRole) for item in items]
+            context.fire('event::DataNode::Select::items', items_nodes)
+
+    
+    def mousePressEvent(self, event):
+        super().mousePressEvent(event)
+        #print("Tree Widget mousePressEvent:", event)
+        if event.button() == Qt.LeftButton:
+            # 判断是否选中某个组件对象
+            # 获取当前选择列表
+            items = self.selectedItems()
+            if len(items) == 0:
+                return 
+
+            nodes = [item.data(0, Qt.UserRole) for item in items]
+            for node in nodes:
+                if not node:
+                    return 
+
+            id_list = [node.get_id() for node in nodes]
+            kind_list = [node.get_kind() for node in nodes]
+
+            id = ";".join(id_list)
+            kind = ";".join(kind_list)
+
+            drag = QDrag(self)
+            mimeData = QMimeData()
+
+            #mimeData.setText("abcd")
+            mimeData.setData('DragNodeID', bytes(id, encoding='UTF-8'))
+            mimeData.setData('DragNodeKind', bytes(kind, encoding='UTF-8'))
+            mimeData.setImageData( nodes)
+            drag.setMimeData(mimeData)
+            #drag.setPixmap(QPixmap("C:/Work/QtUi/goto.png"))
+
+            dropAction = drag.exec(Qt.CopyAction | Qt.MoveAction)
+    
+    def dragEnterEvent(self, event):
+        #if (event->mimeData()->hasFormat("text/plain"))
+        #print("Tree Widget dragEnterEvent:", event)
+        event.acceptProposedAction()
+
+    def dragMoveEvent(self, event):
+        global context
+        #print("Tree Widget dragMoveEvent:", event, event.pos())
+        parent = self.itemAt(event.pos())
+        if not parent:
+            event.ignore()
+            return
+
+        #print('Current Item Rect:', )
+        node = parent.data(0, Qt.UserRole)
+        if not node:
+            event.ignore()
+            return 
+
+        shift_key = False
+        ctrl_key = False
+        alt_key = False
+
+        if event.keyboardModifiers() & Qt.ShiftModifier:
+            shift_key = True
+        if event.keyboardModifiers() & Qt.ControlModifier:
+            ctrl_key = True
+        if event.keyboardModifiers() & Qt.AltModifier:
+            alt_key = True
+
+        mimeData = event.mimeData()
+        drag_nodes = mimeData.imageData()
+        nodeid = bytes(mimeData.data('DragNodeID')).decode('utf-8')
+        nodekind = bytes(mimeData.data('DragNodeKind')).decode('utf-8')
+        if nodeid == "":
+            event.ignore()
+            return 
+
+        kinds = node.get_child_kinds()
+        if shift_key or alt_key:
+            kinds = node.parent().get_child_kinds()
+
+        kind_list = nodekind.split(';')
+
+        b_all_in = True
+        for kind in kind_list:
+            if kind != "" and kind not in kinds:
+                b_all_in = False
+                break
+            
+        if b_all_in:
+            event.acceptProposedAction()
+            return 
+        else:
+            exts = context.find_extension("DataExplore::DataNode::DragAction")
+            for ext in exts:
+                if ext.dragMoveEvent(event, drag_nodes, node) == 1:
+                    event.acceptProposedAction()
+                    return 
+
+        event.ignore()
+        
+        
+
+    def dropEvent(self, event):
+        global context
+        #print("Tree Widget dropEvent:", event)
+        #textBrowser->setPlainText(event->mimeData()->text());
+        #mimeTypeCombo->clear();
+        #mimeTypeCombo->addItems(event->mimeData()->formats());
+
+        # Qt.ShiftModifier
+        # Qt.ControlModifier
+        #print('Keys:', event.keyboardModifiers())
+        #print('Mouse Buttons:', event.mouseButtons())
+
+        mimeData = event.mimeData()
+
+        nodeid = bytes(mimeData.data('DragNodeID')).decode('utf-8')
+        nodekind = bytes(mimeData.data('DragNodeKind')).decode('utf-8')
+        if nodeid == "":
+            event.ignore()
+            return 
+
+        shift_key = False
+        ctrl_key = False
+        alt_key = False
+
+        if event.keyboardModifiers() & Qt.ShiftModifier:
+            shift_key = True
+        if event.keyboardModifiers() & Qt.ControlModifier:
+            ctrl_key = True
+        if event.keyboardModifiers() & Qt.AltModifier:
+            alt_key = True
+
+        parent = self.itemAt(event.pos())
+        if not parent:
+            event.ignore()
+            return
+
+        node = parent.data(0, Qt.UserRole)
+        if not node:
+            event.ignore()
+            return 
+
+        drag_node = mimeData.imageData()
+
+        for node_t in drag_node:
+            if node.get_id() == node_t.get_id():
+                return 
+
+        kinds = node.get_child_kinds()
+        if shift_key or alt_key:
+            kinds = node.parent().get_child_kinds()
+
+        kind_list = nodekind.split(';')
+
+        b_all_in = True
+        for kind in kind_list:
+            if kind != "" and kind not in kinds:
+                b_all_in = False
+                break
+
+        if b_all_in:
+            if ctrl_key:
+                drag_node = [node_t.clone() for node_t in drag_node]
+            else:
+                for node_t in drag_node:
+                    node_t.parent().remove_child(node_t)
+
+            if shift_key:
+                # 前方插入
+                for node_t in drag_node:
+                    node.parent().insert_child(node_t, node)
+            elif alt_key:
+                # 后方插入
+                for node_t in drag_node:
+                    node.parent().insert_child_after(node_t, node)
+            else:
+                # 子节点插入
+                for node_t in drag_node:
+                    node.add_child(node_t)
+        else:
+            exts = context.find_extension("DataExplore::DataNode::DragAction")
+            for ext in exts:
+                ext.dropEvent(event, drag_node, node)
+
+        event.acceptProposedAction()
+
+class DataExploreView(QDockWidget):
+    def __init__(self):
+        super().__init__()
+
+        self._open = None
+        self._menu = None
+
+        self.treeWidget = DragTreeWidget()
+        self.treeWidget.setDragEnabled(True)
+        self.treeWidget.setAcceptDrops(True)
+        self.treeWidget.setDefaultDropAction(Qt.MoveAction);
+        self.treeWidget.setDragDropMode(QAbstractItemView.InternalMove)
+        self.treeWidget.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.treeWidget.setColumnCount(2)
+        self.treeWidget.setHeaderLabels(['Name', 'Kind'])
+        self.treeWidget.setColumnWidth(0, 280)
+        self.treeWidget.setMinimumWidth(360)
+        self.treeWidget.setSelectionMode(QAbstractItemView.ExtendedSelection)
+        
+
+        # 右键菜单
+        self.treeWidget.customContextMenuRequested.connect(self.openMenu)
+        # 双击编辑名称
+        self.treeWidget.itemDoubleClicked.connect(self.double_click)
+        # 选择改变
+        #self.treeWidget.itemPressed.connect(self.itemPressed)
+
+        self.setWidget(self.treeWidget)
+    def openMenu(self, position):
+        #item = self.treeWidget.currentItem()
+        items = self.treeWidget.selectedItems()
+        items_nodes = [item.data(0, Qt.UserRole) for item in items]
+
+        # 弹出菜单
+        menu = QMenu()
+        self._menu = menu
+        
+        # 获取类型节点注册的菜单
+        exts = context.find_extension("PL::DataExplore::Menu")
+        sort_exts = sorted(exts, key=lambda ext: ext.priorty())
+        for ext in sort_exts:
+            ext.add_menu(menu, items_nodes)
+
+        menu.exec_(self.treeWidget.viewport().mapToGlobal(position))
+
+    def double_click(self, item, col):
+        node = item.data(0, Qt.UserRole)
+        
+        exts = context.find_extension("PL::Basic::Editor")
+        for ext in exts:
+            if (ext.is_match_data(node)):
+                global _frame
+                tabWidget = _frame.centralWidget ()
+                widget = ext.create_editor(tabWidget)
+                widget.on_init(node)
+                tabWidget.addTab(widget, node.get_name())
+                break
+
+    def itemPressed(self, item, col):
+        if self._open:
+            item = self._open
+            self.treeWidget.closePersistentEditor(self._open,0)
+            self._open = None
+
+            node = item.data(0, Qt.UserRole)
+            #print("name.....:", item.text(0))
+            node.set_name(item.text(0))
+        
+        global context
+        context.fire('event::DataNode::Select', item.data(0, Qt.UserRole))
+
+    def close_project(self):
+        global context
+        #rint("Close project......................")
+        if self.treeWidget.topLevelItemCount() > 0:
+            self.treeWidget.takeTopLevelItem(0)
+
+    def _new_node(self, node, parent = None):
+        global context
+
+        new_item = None
+        if not parent:
+            new_item = QTreeWidgetItem(self.treeWidget)
+        else:
+            new_item = QTreeWidgetItem(parent)
+
         obj = RootNodeCallback(new_item , node)
 
         new_item.setText(0, node.get_name())
@@ -252,29 +462,31 @@ class ActionNewNode:
     def on_new_node(self, value):
         global context
 
-        print("on_new_node")
-
         name, ok = self.get_name(self._kind)
         if not (ok and name):
             return 
+        if len(self._parent) > 1:
+            return 
 
         data_service = context.find_service("DataService")
-        print(self._parent)
-        if not self._parent:
+        if len(self._parent) == 0:
             # 创建根节点
             new_node = data_service.create_rootnode(name)
         else:
             # 创建子节点
             new_node = data_service.create_node(self._kind, name)
-            self._parent.add_child(new_node)
+            self._parent[0].add_child(new_node)
     def on_delete(self, value):
-        print("on_delete:", self._parent)
-        self._parent.delete()
+        for node in self._parent:
+            node.delete()
     def on_rename(self, value):
+        if len(self._parent) != 1:
+            return 
+
         new_name, ok  = self.get_name(self._kind)
         if not (ok and new_name):
             return 
-        self._parent.set_name(new_name)
+        self._parent[0].set_name(new_name)
     def on_copy(self, value):
         global g_copy_obj
         global g_cut_obj
@@ -289,14 +501,19 @@ class ActionNewNode:
         global g_copy_obj
         global g_cut_obj
 
+        if len(self._parent) != 1:
+            return 
+
         new_obj = None
         if g_cut_obj:
-            g_cut_obj.parent().remove_child(g_cut_obj)
+            for node in g_cut_obj:
+                node.parent().remove_child(node)
             new_obj = g_cut_obj
         else:
-            new_obj = g_copy_obj.clone()
+            new_obj = [node.clone() for node in g_copy_obj]
         
-        self._parent.add_child(new_obj)
+        for node in new_obj:
+            self._parent[0].add_child(node)
         g_copy_obj = None
         g_cut_obj = None
 
@@ -305,15 +522,18 @@ class ActionNewNode:
 class DataExploreBasicOpMenu:
     def __init__(self):
         self.menus = []
-    def add_menu(self, menu, node):
+    def priorty(self):
+        return 1000
+    def add_menu(self, menu, nodes):
         self.menus = []
 
-        if node == None:
+        if len(nodes) == 0:
             obj = ActionNewNode("")
             action = menu.addAction("新建根节点2")
             action.triggered.connect(obj.on_new_node)
             self.menus.append(obj)
-        else:
+        elif len(nodes) == 1:
+            node = nodes[0]
             kind = ""
             if node != None:
                 kind = node.get_kind()
@@ -322,7 +542,7 @@ class DataExploreBasicOpMenu:
             for k in kinds:
                 action = menu.addAction("新建"+k)
                 
-                obj = ActionNewNode(k, node)
+                obj = ActionNewNode(k, nodes)
                 self.menus.append(obj)
                 action.triggered.connect(obj.on_new_node)
 
@@ -332,14 +552,11 @@ class DataExploreDeleteMenu:
         self.menus = []
     def priorty(self):
         return 1000
-    def add_menu(self, menu, node):
+    def add_menu(self, menu, nodes):
         self.menus = []
 
-        if node == None:
-            return
-        else:
-            kind = node.get_kind()
-            obj = ActionNewNode(kind, node)
+        if len(nodes) > 0:
+            obj = ActionNewNode("", nodes)
             self.menus.append(obj)
             action = menu.addAction("删除")
             action.triggered.connect(obj.on_delete)
@@ -349,14 +566,11 @@ class DataExploreRenameMenu:
         self.menus = []
     def priorty(self):
         return 1000
-    def add_menu(self, menu, node):
+    def add_menu(self, menu, nodes):
         self.menus = []
 
-        if node == None:
-            return
-        else:
-            kind = node.get_kind()
-            obj = ActionNewNode(kind, node)
+        if len(nodes) == 1:
+            obj = ActionNewNode("", nodes)
             self.menus.append(obj)
             action = menu.addAction("重命名")
             action.triggered.connect(obj.on_rename)
@@ -366,14 +580,11 @@ class DataExploreCopyMenu:
         self.menus = []
     def priorty(self):
         return 1000
-    def add_menu(self, menu, node):
+    def add_menu(self, menu, nodes):
         self.menus = []
 
-        if node == None:
-            return
-        else:
-            kind = node.get_kind()
-            obj = ActionNewNode(kind, node)
+        if len(nodes)>0:
+            obj = ActionNewNode("", nodes)
             self.menus.append(obj)
             action = menu.addAction("复制")
             action.triggered.connect(obj.on_copy)
@@ -383,14 +594,11 @@ class DataExploreCutMenu:
         self.menus = []
     def priorty(self):
         return 1000
-    def add_menu(self, menu, node):
+    def add_menu(self, menu, nodes):
         self.menus = []
 
-        if node == None:
-            return
-        else:
-            kind = node.get_kind()
-            obj = ActionNewNode(kind, node)
+        if len(nodes) > 0:
+            obj = ActionNewNode("", nodes)
             self.menus.append(obj)
             action = menu.addAction("剪切")
             action.triggered.connect(obj.on_cut)
@@ -400,10 +608,10 @@ class DataExplorePasteMenu:
         self.menus = []
     def priorty(self):
         return 1000
-    def add_menu(self, menu, node):
+    def add_menu(self, menu, nodes):
         self.menus = []
 
-        if node == None:
+        if len(nodes) != 1:
             return
 
         global g_copy_obj
@@ -412,21 +620,21 @@ class DataExplorePasteMenu:
         if g_copy_obj == None and g_cut_obj == None:
             return 
 
-        kind = ''
+        kind_ = ''
         kinds = []
         if g_copy_obj:
-            kind = g_copy_obj.get_kind()
+            kind_ = [node.get_kind() for node in g_copy_obj]
         elif g_cut_obj:
-            kind = g_cut_obj.get_kind()
-        kinds = node.get_child_kinds()
+            kind_ = [node.get_kind() for node in g_cut_obj]
+        kinds = nodes[0].get_child_kinds()
 
-        if kind not in kinds:
-            return 
+        for kind in kind_:
+            if kind not in kinds:
+                return 
         #print("copy kind:", kind)
         #print('Child kinds:', kinds)
 
-        kind = node.get_kind()
-        obj = ActionNewNode(kind, node)
+        obj = ActionNewNode("", nodes)
         self.menus.append(obj)
         action = menu.addAction("粘贴")
         action.triggered.connect(obj.on_paste)
@@ -495,7 +703,6 @@ class PaneExtensionToolbar:
 
 def OnOpenProject(event, node):
     global _dataexplore_view
-    print(".------------------")
     _dataexplore_view.on_project_open(node)
 
 def OnRootNodeNew(event, node):
@@ -515,13 +722,26 @@ class DataExploreMenu:
         pass
 '''
 
+DataExploreDragActionExtDef = '''
+class DataExploreDragActionExtDef:
+    def dragMoveEvent(self,event, node, target_node):
+        return 1000
+    def dropEvent(self, event, node, target_node):
+        pass
+'''
+
 config = {
     'pluginid': 'UI::Core::DataExplore',
     "extensions_def" : [
         {
             "name": "PL::DataExplore::Menu",
             "define": DataExploreMenu
+        },
+        {
+            "name": "DataExplore::DataNode::DragAction",
+            "define": DataExploreDragActionExtDef
         }
+        
     ],
     "extensions" : [
         {
